/* libcmdir_record.c
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "libcmdir.h"

int execute_commandir_record_arg(int argc, char *argv[]) ;
extern unsigned char standalone;
extern struct commandir_tx_order * ordered_commandir_devices;
extern struct commandir_tx_order * first_commandir_device;
extern char * rec_filename;
extern int gap_size;
extern unsigned char displayOnly;
extern unsigned char pipeOnly;

int commandir_record_cmd(char * cmdline)
{
  char *ptrIdx[1024];
  int ptrs = 0, x;
  int arg_len;
  
  arg_len = strlen(cmdline);
  printf("Processing cmd line (%d): %s\n", arg_len, cmdline);
  
  // Process cmdline args
  // Intentionally ignoring the first arg, which is the type of command
  ptrIdx[ptrs++] = cmdline; // we ignore the first one, but have to keep it consistent with calling direct
  for(x=0;x<arg_len-1;x++)
  {
    if(cmdline[x]==' ') {
      // Zero terminate, instead of space seperate
      ptrIdx[ptrs++] = &cmdline[x+1];
      cmdline[x] = '\0';
    }
  }
  
  if(ptrs > 0) {
    printf("Detected %d command line args\n", ptrs);
    return execute_commandir_record_arg(ptrs, ptrIdx);
  }
  return 0;
}



int execute_commandir_record_arg(int argc, char *argv[]) 
{
  int x;
  struct commandir_device * pcd = NULL;

  // Init the CommandIR, then record each of the names provided:

  if(standalone == 1) {
    hardware_scan();
    hardware_setorder();
    if(first_commandir_device == NULL)
    {
      printf("No CommandIRs detected\n");
      return 0;
    }
  }
  
  for(x=1; x<argc; x++)
  {
    if(argv[x][0] == '-')
    {
      switch(argv[x][1])
      {
        case 's':
          // Named CommandIR - seach for this one
          pcd = getCommandIR_By_Name(&argv[x][2]);
          if(!pcd) {
            printf("CommandIR %s not found!\n", &argv[x][2]);
            return -1;
          }
          printf("Recording from %s.\n", &argv[x][2]);
          break;
        case 'g':
          gap_size = atoi(&argv[x][2]);
          if(gap_size < 1000)
          {
            printf("Please specify a larger gap size: %d is too small\n", gap_size);
            return -1;
            break;
          }
          printf("Setting gap size to %d\n", gap_size);
          break;
        case 'd':
          // Display Only - don't save to a file
          displayOnly = 1;
          break;
        default:
          printf("Unknown command line argument %c\n", argv[x][1]);
      }
    } else {
      if(displayOnly) {
        printf("Do not specify a filename if you want to display codes with -d.\n");
        return -1;
      }
      if(pipeOnly) {
        printf("Do not specify a filename if you want to pipe with -p.\n");
        return -1; 
      }
      rec_filename = argv[x];
      printf("Recording %s ...\n", rec_filename);
      if(pcd)
        commandir_rec(pcd->cmdir_udev);
      else
        commandir_rec(NULL);
    }
  }
  
  if(displayOnly | pipeOnly) {
    if(pcd)
      commandir_rec(pcd->cmdir_udev);
    else
      commandir_rec(NULL);
  }
  

}
